// AEncodeProperties.h: interface for the AEncodeProperties class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(_AENCODEPROPERTIES_H__INCLUDED_)
#define _AENCODEPROPERTIES_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <windows.h>

#include "BladeMP3EncDLL.h"

class AEncodeProperties  
{
public:
	AEncodeProperties() {}
	virtual ~AEncodeProperties() {}

	inline const bool GetCopyrightMode() const { return bCopyright; }
	inline const bool GetCRCMode() const { return bCRC; }
	inline const bool GetOriginalMode() const { return bOriginal; }
	inline const bool GetPrivateMode() const { return bPrivate; }
	inline const bool GetNoBiResMode() const { return bNoBitRes; }
	inline const bool GetForceChannelMode() const { return bForceChannel; }
	inline const bool GetVBRUseMode() const { return bVBRused; }
	inline const bool GetXingFrameMode() const { return bXingFrame; }
	inline const bool GetResampleMode() const { return bResample; }
	
	const unsigned int GetBitrateValue() const;
	const int GetBitrateValue(DWORD & bitrate, const DWORD MPEG_Version) const;
	const int GetBitrateValueMPEG2(DWORD & bitrate) const;
	const int GetBitrateValueMPEG1(DWORD & bitrate) const;
	inline const int GetBitrateString(char * string, int string_size) const {return GetBitrateString(string,string_size,nMinBitrateIndex); }
	const int GetBitrateString(char * string, int string_size, int a_bitrateID) const;
	inline const int GetBitrateLentgh() const { return sizeof(the_Bitrates) / sizeof(unsigned int); }
	inline const unsigned int GetResampleFreq() const { return the_SamplingFreqs[nSamplingFreqIndex]; }
	bool GetVBRValue(DWORD & MaxBitrate, int & Quality, BOOL & VBRHeader, const DWORD MPEG_Version) const;
	char * GetDllLocation() const { return (char *)DllLocation; }
	const unsigned int GetDllLocationSize() const { return sizeof(DllLocation); }
	char * GetOutputDirectory() const { return (char *)OutputDir; }

	const unsigned int GetChannelModeValue() const;
	const char * GetChannelModeString(const int a_channelID) const;
	inline const char * GetChannelModeString() const {return GetChannelModeString(nChannelIndex); }
	inline const int GetChannelLentgh() const { return sizeof(the_ChannelModes) / sizeof(unsigned int); }

	const LAME_QUALTIY_PRESET GetPresetModeValue() const;
	const char * GetPresetModeString(const int a_presetID) const;
//	inline const char * GetChannelModeString() const {return GetChannelModeString(nChannelIndex); }
//	inline const int GetPresetLentgh() const { return sizeof(the_Presets) / (sizeof(char) * 13); }
	inline const int GetPresetLentgh() const { return sizeof(the_Presets) / sizeof(LAME_QUALTIY_PRESET); }

	inline void SetCopyrightMode(const bool bMode) { bCopyright = bMode; }
	inline void SetCRCMode(const bool bMode) { bCRC = bMode; }
	inline void SetOriginalMode(const bool bMode) { bOriginal = bMode; }
	inline void SetPrivateMode(const bool bMode) { bPrivate = bMode; }
	inline void SetNoBiResMode(const bool bMode) { bNoBitRes = bMode; }
	inline void SetForceChannelMode(const bool bMode) { bForceChannel = bMode; }
	inline void SetVBRUseMode(const bool bMode) { bVBRused = bMode; }
	inline void SetXingFrameMode(const bool bMode) { bXingFrame = bMode; }

	bool Config(const HINSTANCE Hinstance, const HWND HwndParent);
	bool InitConfigDlg(HWND HwndDlg);
	bool UpdateValueFromDlg(HWND HwndDlg);
	void ParamsSave(void);
	void ParamsRestore(void);

	static void DisplayVbrOptions(HWND hDialog, bool bValue);
//	int VbrQuality;
protected:
	bool bCopyright;
	bool bCRC;
	bool bOriginal;
	bool bPrivate;
	bool bNoBitRes;
	bool bVBRused;
	bool bXingFrame;
	bool bForceChannel;
	bool bResample;

	int VbrQuality;

	static const unsigned int the_ChannelModes[4];
	int nChannelIndex;

	static const unsigned int the_Bitrates[18];
	static const unsigned int the_MPEG1_Bitrates[14];
	static const unsigned int the_MPEG2_Bitrates[14];
	int nMinBitrateIndex; // CBR and VBR
	int nMaxBitrateIndex; // only used in VBR mode

	static const unsigned int the_SamplingFreqs[9];
	int nSamplingFreqIndex;

	static const LAME_QUALTIY_PRESET the_Presets[15];
	int nPresetIndex;

	char DllLocation[512];
	char OutputDir[MAX_PATH];

	HINSTANCE hDllInstance;
private:
};

#endif // !defined(_AENCODEPROPERTIES_H__INCLUDED_)
