/************************************************************************
Project               : C++ parameters saving class
File version          : 0.1

BSD License post 1999 : 

Copyright (c) 2001, Steve Lhomme
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met: 

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO 
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE. 
************************************************************************/

#include <assert.h>

#include "AParameters.h"

AParameters::AParameters()
	:my_base_mode(HKEY_LOCAL_MACHINE)
	,my_base_key(NULL)
	,my_subkey_current_index(0)
	,my_working_key(NULL)
{
}

AParameters::AParameters(const HKEY the_base)
	:my_base_mode(HKEY_LOCAL_MACHINE)
	,my_base_key(the_base)
	,my_subkey_current_index(0)
	,my_working_key(NULL)
{
}

AParameters::~AParameters()
{
	assert(my_base_key == NULL);
	assert(my_working_key == NULL);
}

bool AParameters::Attach(const char * the_base, const REGSAM the_access_mode)
{
	assert(my_base_key == NULL);
	assert(my_working_key == NULL);

	bool bResult = false;

	if (::RegOpenKeyEx( my_base_mode, the_base, 0, the_access_mode , &my_base_key ) != ERROR_SUCCESS)
	{
		my_base_key = NULL;
	}
	else
	{
		my_working_key = NULL;
		my_subkey_current_index = 0;
		bResult = true;
	}

	return bResult;
}

void AParameters::Detach()
{
	assert(my_base_key != NULL);
//	assert(my_working_key == NULL);

	if (my_working_key != NULL)
	{
		::RegCloseKey(my_working_key);
		my_working_key = NULL;
	}

	::RegCloseKey(my_base_key);
	my_base_key = NULL;
}

void AParameters::ResetSubKey()
{
	my_subkey_current_index = 0;
}

AParameters::KEY_RETURNS AParameters::NextSubKey(char * the_buffer, unsigned int the_buffer_size)
{
	assert( (my_working_key != NULL) || (my_base_key != NULL));

	KEY_RETURNS sResult = KEY_ERROR;
	LONG Result;

	if (my_subkey_current_index != -1)
	{
		if (my_working_key != NULL)
			Result = ::RegEnumKey( my_working_key, my_subkey_current_index, the_buffer, the_buffer_size);
		else
			Result = ::RegEnumKey( my_base_key,    my_subkey_current_index, the_buffer, the_buffer_size);
			
		if (Result == ERROR_NO_MORE_ITEMS)
		{
			my_subkey_current_index = -1;
			sResult = LAST_KEY;
		}
		else if (Result != ERROR_SUCCESS)
		{
			my_subkey_current_index = -1;
		}
		else
		{
			my_subkey_current_index++;
			sResult = KEY_FOUND;
		}
	}

	return sResult;
}

/*
bool AParameters::SelectSubKey(const char * the_subkey, const bool bCreateKey, const REGSAM the_access_mode)
{
	assert( my_base_key != NULL );
//	assert( my_working_key == NULL );

	bool bResult = false;
	LONG Result;

	if (my_working_key != NULL)
	{
		::RegCloseKey(my_working_key);
		my_working_key = NULL;
	}

	if (bCreateKey)
		Result = ::RegCreateKeyEx( my_base_key, the_subkey, 0, "", REG_OPTION_NON_VOLATILE, the_access_mode , NULL, &my_working_key, NULL );
	else
		Result = ::RegOpenKeyEx( my_base_key, the_subkey, 0, the_access_mode , &my_working_key );

	if (Result != ERROR_SUCCESS)
	{
		my_working_key = NULL;
	}
	else
	{
		// my_base_key = my_old_working_key
		my_subkey_current_index = 0;
		bResult = true;
	}

	return bResult;
}
*/

bool AParameters::CreateSubKey(const char * the_subkey, const bShouldNotExist)
{
	bool bResult = false;
	LONG Result;
	HKEY tmpKey;
	DWORD existed;

	if (my_working_key != NULL)
		Result = ::RegCreateKeyEx( my_working_key, the_subkey, 0, "", REG_OPTION_NON_VOLATILE, KEY_CREATE_SUB_KEY , NULL, &tmpKey, &existed );
	else
		Result = ::RegCreateKeyEx( my_base_key,    the_subkey, 0, "", REG_OPTION_NON_VOLATILE, KEY_CREATE_SUB_KEY , NULL, &tmpKey, &existed );

	if (Result == ERROR_SUCCESS)
	{
		::RegCloseKey(tmpKey);

		if (!bShouldNotExist || (existed == REG_CREATED_NEW_KEY))
			bResult = true;
	}

	return bResult;
}

bool AParameters::DeleteKey(const char * the_key)
{
	return false;
}

void AParameters::QueryValue(const char * the_data_name, void * the_data, unsigned long & the_buffer_size)
{
	assert( (my_working_key != NULL) || (my_base_key != NULL));

	LONG Result;

/*{
char tst[500];
wsprintf(tst,"the_data_name = %s / Result = %08X / the_buffer_size = %d",the_data_name,Result,the_buffer_size);
OutputDebugString(tst);
}*/
	if (my_working_key != NULL)
	{
//OutputDebugString("use my_working_key");
		Result = ::RegQueryValueEx(my_working_key, the_data_name, NULL, NULL, (BYTE *)the_data, &the_buffer_size );
	}
	else
	{
//OutputDebugString("use my_base_key");
		Result = ::RegQueryValueEx(my_base_key,    the_data_name, NULL, NULL, (BYTE *)the_data, &the_buffer_size );
	}

/*{
char tst[500];
wsprintf(tst,"the_data_name = %s / Result = %08X / the_buffer_size = %d",the_data_name,Result,the_buffer_size);
OutputDebugString(tst);
}*/
	if (Result != ERROR_SUCCESS)
	{
		the_buffer_size = 0;
	}
}


AParameters * AParameters::GetSubKey(const char * the_subkey, const bool bCreateKey, const REGSAM the_access_mode)
{
	assert( my_base_key != NULL );
//	assert( my_working_key == NULL );

	LONG Result;
	
	AParameters * tmpParam = NULL;
	HKEY tmpKey;
/*
	if (my_working_key != NULL)
	{
		::RegCloseKey(my_working_key);
		my_working_key = NULL;
	}
*/
	if (my_working_key != NULL)
	{
		if (bCreateKey)
			Result = ::RegCreateKeyEx( my_working_key, the_subkey, 0, "", REG_OPTION_NON_VOLATILE, the_access_mode , NULL, &tmpKey, NULL );
		else
			Result = ::RegOpenKeyEx( my_working_key, the_subkey, 0, the_access_mode , &tmpKey );
	}
	else
	{
		if (bCreateKey)
			Result = ::RegCreateKeyEx( my_base_key, the_subkey, 0, "", REG_OPTION_NON_VOLATILE, the_access_mode , NULL, &tmpKey, NULL );
		else
			Result = ::RegOpenKeyEx( my_base_key, the_subkey, 0, the_access_mode , &tmpKey );
	}

	if (Result != ERROR_SUCCESS)
	{
		my_working_key = NULL;
	}
	else
	{
		tmpParam = new AParameters(tmpKey);

		// my_base_key = my_old_working_key
//		my_subkey_current_index = 0;
//		delete tmpParam;
//		tmpParam = NULL;
	}

	return tmpParam;
}

void AParameters::ReleaseSubKey(AParameters * the_sub_key) const
{
	the_sub_key->Detach();
	delete the_sub_key; 
}
